# -*- coding: utf-8 -*-
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
__doc__="""ATCTMinimumSample Product for Plone (ATCT)
$Id: smallcontent.py 137 2006-05-06 10:58:50Z taka $"""

__version__='$Rev$'[11:-2]
__product_version__ = "Small 0.0.1"
__author__  = 'Shimizukawa'
__docformat__ = 'plaintext'


# Import modules and functions, etc. used in the following codes. 
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder
from Products.Archetypes.public import Schema,registerType
from Products.ATContentTypes.content.base import ATCTFolder
from Products.ATContentTypes.content.folder import finalizeATCTSchema
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import IntDisplayList

# Fields used with this product.
from Products.Archetypes.public import StringField,IntegerField,\
                                   LinesField,ReferenceField,BooleanField
# Widgets used with this product.
from Products.Archetypes.public import StringWidget,TextAreaWidget,\
                               IntegerWidget,SelectionWidget,BooleanWidget,\
                               ReferenceWidget

# Basic set of values used with this product.
from Products.ATCTSmallSample.config import *

# The type of data and the method of displaying UI used by this contents 
# type are defined as a schema. The schema of a succession former class 
# is copied, and it enhances it originally instead of make by full 
# scratch. If you make a product as compatible to ATCT, you will enhance 
# ATContentTypes.content.schemata.ATContentTypeSchema.
# You will get details of Fileds and Widgets, see 
# http://plone.org/products/archetypes/documentation/manual/quickref
# that describe some parameters.
# However, newest documents are always with the code. It is very useful 
# to read Archetypes/Field.py and Widget.py.
SmallContentSchema = ATCTFolder.schema.copy() + Schema((

    # Reference to another object on Plone
    ReferenceField(# Id used as variable identifier name
                   'rel_val',
                   relationship='any_rel',
                   # If you limit kind of the object to display, you spedify portal_type by the tupple.
                   allowed_types=(),
                   # Can the plural be selected?
                   multivalued=0,
                   # Is it an indispensable item?
                   required=True,
                   widget=ReferenceWidget(# Name to display on UI
                                          label='Reference',
                                          # TipHelp to display on UI
                                          description='Link to referenced object on Plone')
                 ),

    # True or False is preserved.
    BooleanField('bool_val',
                 # Default value when newly making a object. When the schema is added back, it is applied to an existing object.
                 default=False,
                 required=True,
                 widget=BooleanWidget(label='True/False',
                                      description='True or False value is maintained.',
                                      ),
                 ),

    IntegerField('stock',
                 default=0,
                 required=True,
                 widget=IntegerWidget(label='Int value(free input)',
                                      description='An arbitrary numerical value is freely input.',
                                      ),
                 ),

    IntegerField('int_val',
                 default=0,
                 # Choices used for the selection type are given.
                 # DisplayList and IntDisplayList are initialized by list of tuple.
                 # The tuples are composed of the VALUE is used to HTML form value
                 # and STRING is used to display. [(VALUE, STRING), ...]
                 vocabulary=IntDisplayList([(i, i) for i in range(0, 101)]),
                 widget=SelectionWidget(label='Int value(selection)',
                                        description='select value from 0 to 100.',
                                        # The display form type(DropDown,Radio) is
                                        # specified. "format='flex'" is specified,
                                        # form type is automatically decided from
                                        # the number of choices. 
                                        format='select',
                                        ),
                 ),

    ))




#Finalize schema definition
# This process add peculiar schemas 'Allow Discussion on this item' and 
# 'Related content' to ATCT.
# See ATContentTypes.content.schemata.finalizeATCTSchema.
finalizeATCTSchema(SmallContentSchema)


# The class that becomes new contents type that succeeds to ATCTFolder is 
# defined. This contents type can store the object of other contents 
# types etc. in the container like the folder because it succeeds to 
# ATCTFolder.
class SmallContent(ATCTFolder):
    """
        create SmallContent instance.

    >>> from Products.ATCTSmallSample.content.smallcontent import SmallContent
    >>> small = SmallContent('small')
    >>> small.meta_type
    'SmallContent'

        test IInlineObject interface existing.

    >>> from Products.COREBlog2.interfaces.IInlineObject import IInlineObject
    >>> bool(IInlineObject.isImplementedBy(small))
    True

    """
    # Above doc-string is a manual and a test code used for DocTest. 
    # DocTest of the class is use example help as it is. 
    # A big effect is demonstrated in the class for utility though etc.
    # It is ATCTSmallSample/tests/testDocTest.py to execute the 
    # above doc-string as DocTest. 

    # Hereafter, a necessary value is defined as a class variable.

    # The schema definition is registered.
    schema         = SmallContentSchema

    # The icon displayed in navigation and an additional menu, etc. is 
    # specified. The icon image is put on the place in which it can 
    # access it from anywhere. (i.e. under skins/ATCTSmallSample/)
    content_icon   = 'smallcontent.gif'

    # Meta type of Zope. Because the problem of no start of Zope etc. 
    # when this name overlaps on the Zope instance occurs, making to a 
    # long to some degree name might be good. Moreover, the same name as 
    # meta_type as for portal_type and archetype_name is recommended to 
    # be set to avoid confusing.
    meta_type      = 'SmallContent'

    # Meta type of Plone. Name specified for invokeFactory() when 
    # instance of content is added on Plone. The portal_type name has not 
    # been changed though it shifts to mounting the Archetypes base when 
    # Plone becomes 2.1, and Document etc. became meta_type named 
    # ATDocument. meta_type==Document that it is existing on Zope loses 
    # the influence in losing the influence on the object, and not 
    # changing portal_type in changing meta_type the product for existing 
    # Plone.
    portal_type    = 'SmallContent'

    # Archetype name. It is a character string to be translated i18n by 
    # the name displayed in the instance addition menu on UI of Plone. 
    # Because the influence on an existing instance etc. is large when 
    # meta_type and portal_type are rewritten back, it is prepared to use 
    # it as a display name.
    archetype_name = 'SmallContent'

    #
    immediate_view = 'base_view'

    # View is specified in the early the case to enable two or more of 
    # View selections with suppl_views.
    default_view   = 'base_view'

    # The method name and the PageTemplate name that becomes choices are 
    # enumerated by the tupple when assuming that two or more View to 
    # display contents can be selected (Because it is necessary to 
    # specify the object name for PageTemplate, the extension is 
    # unnecessary). 
    suppl_views    = ()

    #
    typeDescription= 'A small content.'

    #
    typeDescMsgId  = 'small_content_description'

    #
    assocMimetypes = ()

    #
    assocFileExt   = ()

    #
    cmf_edit_kws   = ()

    # The BOOL value my whether the instance may being to be made for an 
    # arbitrary place on the Plone site is specified. It is not displayed 
    # that False is specified in an additional menu in an arbitrary 
    # place. If portal_type of this contents is specified for 
    # allowed_content_types in other contents types, adding it to the 
    # contents type right under becomes possible. Or, the thing assumed 
    # to be a contents type that wants to be added only on the program 
    # code can be done.
    global_allow   = True

    # The BOOL value whether the subcontents being able to be added is 
    # specified when contents are folderlish (container type) classes. 
    # The contents type that can be added that False is set is not 
    # limited. When True is set, it comes to be able to register only the 
    # contents type specified for allowed_content_types in the 
    # subcontents.
    filter_content_types = False

    # When filter_content_types is set to True, it is effective. Please 
    # refer to the explanation of filter_content_types for details.
    allowed_content_types = ()

    # In a standard setting of Plone-2.1, Form that specifies id when 
    # instancing new is made is not displayed. Therefore, id is generated 
    # automatically, and when the instance is newly made, id is generated 
    # from the value of the title property automatically when True is set 
    # to __ at_rename_after_creation. It generates it without using the 
    # title property automatically for False. When id is specified at the 
    # setting that specifies id, automatic generation is not done to use 
    # specified id.
    _at_rename_after_creation = True

    # The mounted interface is defined. INonStructuralFolder is 
    # classified into the marker interface that makes the folder type 
    # class not treat as a folder on PloneUI (The contents tab is not 
    # displayed). 
    # It adds it to the interface definition of a succession former class.
    __implements__ = (ATCTFolder.__implements__, INonStructuralFolder, )

    # implement for coreblog IInlineObject interface
    try:
        # The IInlineObject interface that COREBlog2 defines is added to 
        # __imprements__. When COREBlog2 is not in Products, the code is 
        # executed without trouble though damage from cold weather 
        # occurs. It comes to be able to display in-line in contents 
        # (COREBlog2 etc.) that can treat the IInlineObject mounting 
        # object by mounting the method of the IInlineObject interface.
        from Products.COREBlog2.interfaces.IInlineObject import IInlineObject
        __implements__ += (IInlineObject,)
    except:
        pass

    # It is utility to set the right of access of each method of the 
    # class. It doesn't use it because there was no method that should be 
    # made non-Public in SmallContent.
    security = ClassSecurityInfo()

    # In Plone-2.1, the default view of the folder can be specified. In a 
    # word, View of the object that can become a default view can be 
    # displayed instead of the default operation of displaying the object 
    # list in the folder when there are no index_html etc. 
    # (CMFContentPanels etc.). It comes to be able to set it to the 
    # default view by assuming Caene of this function to be True. 
    def canSetDefaultPage(self):
        """Can this content set default page for folder?"""
        return False

    # It is a method of IInlineObject. When True is returned, the object 
    # is displayed in the call former contents body part. It is necessary 
    # to offer PageTemplate name etc. necessary to display in-line by the 
    # getInlineView method. When False is returned, it becomes an 
    # attached display, and it is displayed in the end of contents. It is 
    # necessary to offer the PageTemplate name etc. where the method of 
    # the display is described by the getAttachementView method as well 
    # as the case of True.
    def isInlineObject(self):
        """Is this content inline object or not?

        >>> small.isInlineObject()
        True

        """
        return True

    # It is a method of IInlineObject. When True is returned with 
    # isInlineObject(), it is called. The macro name of PageTemplate and 
    # metal for a display necessary to display in-line is returned by the 
    # tupple.
    def getInlineView(self):
        """For IInlineObject's inline view, if isInlineObject return True.

        >>> small.getInlineView()
        ('smallcontent_inline_view', 'inline_view')

        """

        templ = 'smallcontent_inline_view'
        macro = 'inline_view'
        return templ, macro

    # It is a method of IInlineObject. When False is returned with 
    # isInlineObject(), it is called. An attached display returns the 
    # macro name of PageTemplate and metal for a display necessary to do 
    # by the tupple.
    def getAttachmentView(self):
        """For IInlineObject's attachement view, if isInlineObject return False.

        >>> small.getAttachmentView() is None
        True

        """
        pass


# It is a contents type registration function to Archetypes from which 
# this module is executed when import is done. The project name is 
# acquired with listTypes() with the class registered here in acquisition 
# in ATCTSmallSample/__init__.py with process_types(), and 
# Extentions/install.py.
registerType(SmallContent, PROJECTNAME)

