# -*- coding: utf-8 -*-
#$Id: ATCTSSTestCase.py 138 2006-05-08 16:02:40Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#
# ATCTSSTestCase test setup
#
# ATCTSSTestCase.py describes the common implementation used in all tests
# (SS is abbreviation of SmallSample).


from Products.PloneTestCase import PloneTestCase


# To do this module in import, each test implementation module need not 
# mount the following installation processing again.

# It is necessary to install on the execution of the test to test 
# Zope/Plone and to install (this product depends) product in virtual 
# Zope instance. Because basic products bundled to Zope and Plone are 
# automatically installed, only ATCTSmallSample is installed in here 
# specifying it. 
PloneTestCase.installProduct('ATCTSmallSample')

# Next, "ATCTSmallSample" is set up to virtual Plone environment with 
# setupPloneSite(). This is synonymous with the addition by "Add/Remove 
# Products" of Plone UI. Basic product necessary for operating Plone is 
# automatically installed with PloneTestCase/setup.py, then 
# ATCTSmallSample is installed specifying it. 
PloneTestCase.setupPloneSite(products=['ATCTSmallSample',])




# All test implementation classes will inherit from PloneTestCase class. 
# A necessary initialization and settlement at each test can be done by 
# implementation functions such as afterSetUp() and beforeTearDown() in 
# the test implementation class, and there is case where the utility 
# function that wants to be used in other various tests is needed a lot. 
# Then, the middle base class that inherit from PloneTestCase is prepared 
# and the utility function is prepared here. Each test implementation 
# class will inherit from this middle class.
class ATCTSSTestCase(PloneTestCase.PloneTestCase):

    # The name often used in each test implementation class is prepared 
    # as a class variable.
    defaultTitle = 'Default Testing Title'

    # In generally UnitTest, the initialization function is setUp, and 
    # the settlement function is tearDown. However, ZopeTestCase uses 
    # these for the preparation and the rollback of the environment.
    # Therefore, when the test code that uses ZopeTestCase and 
    # PloneTestCase is made, afterSetUp and beforeTearDown are used 
    # instead of these. Please refer to 
    # lib/python/Testing/ZopeTestCase/doc/TIMELINES.txt bundled to 
    # ZopeTestCase-0.9.8 (ZopeTestCase since or Zope-2.9.0) for details 
    # for the called timing.
    def afterSetUp(self):
        # After the Plone environment is made, the SmallContent instance 
        # is made by the default name. 'self.folder' specified for a 
        # making position indicates an personal folder.
        self._createSmallContent(self.folder)

    # It is a utility function that makes the SmallContent instance used 
    # together.
    def _createSmallContent(self, folder, id='small', title=defaultTitle):
        """Creates and returns a refence to a SmallContent.
        This method publishes a SmallContent instance under folder.  It fills in
        all of the standard properties."""

        # To make the instance, the invokeFactory method of the folder 
        # for making is called.
        folder.invokeFactory(# The first argument specifies portal_type   
                             # registered in portal_types. It is not 
                             # meta_type.
                             'SmallContent',
                             # ID of the object is specified.
                             id=id,
                             # Title of the object is specified.
                             title=title,
                             # Description of the object is specified.
                             description='A SmallContent instance for unit tests.',
                            )

        # The made SmallContent instance is acquired from the folder. 
        small = getattr(folder, id)
        # The setting of initial state of workflow is applied to the made instance.
        self.portal.portal_workflow.doActionFor(small, 'submit')
        # The made instance object is returned.
        return small

    # It is a utility function that uninstalls the product. This function 
    # is used for the uninstallation test etc. of the product. It is used 
    # with testSetup.TestUninstallSmallContent for ATCTSmallSample.
    def uninstallProduct(self, product_name):
        # Removes a product
        tool = getattr(self.portal, 'portal_quickinstaller')
        if tool.isProductInstalled(product_name):
            tool.uninstallProducts([product_name])

