# -*- coding: utf-8 -*-
#$Id: testDocTest.py 138 2006-05-08 16:02:40Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#
# ATCTSSTestCase DocTests
#

# In this test module, test suite that executes DocTest described in an 
# arbitrary module is made.
#
# DocTest is a test that interprets and executes the part of the line 
# that starts by '>>>' like Python interactive mode as ``test execution 
# code`` among DocString of Python (character string not substituted for 
# the variable). Especially, if DocTest is described in DocString that 
# doesn't depend on environment, help of the usage of the function can be 
# held concurrently. Document, Test and Implementation  are consolidated 
# in one place.
#
# Please refer to 
# lib/python/Testing/ZopeTestCase/zopedoctest/ZopeDocTest.txt bundled to 
# ZopeTestCase-0.9.8 (or bundled since Zope-2.9.0) for details. 
# (ZopeDocTest.txt is DocTest, and it is executed from testZopeDocTest.py)
#
# Please note it in the order of executing DocTest. When ZopeDocFileSuite 
# is used, DocString is executed in the following order:
#
#   1. DocString of setUp function set with
#      ZopeDocFileSuite('example.py', setUp=setUp)
#   2. All DocString in example.py
#
# SetUp/tearDown of each DocString of method in each module set by the 
# above-mentioned is not called, and all DocString in the file is 
# executed on regardless of the class or the function, etc. sequentially.
# 
# All DocString in the file is sequentially executed on regardless of the 
# class or the function, etc. setUp/tearDown functions are not called at 
# each test of method in each module. 


# The explanation is omitted about the same to testSetup.py code. Please 
# refer to testSetup.py for details.

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))


# This setUp function is called before DocString of each module is 
# executed. This function does the following two processing.
#
# 1. Preparation for environment such as importAcquisiion necessary to 
#    make instance of contents type class.
# 
# 2. Import COREBlog2 beforehand to prevent DocTest from receiving the 
#    character string output to a standard output at COREBlog2 import.
def setUp(self):
    '''This method will run after the test_class' setUp.

    >>> import Acquisition
    >>> from Globals import INSTANCE_HOME
    '''
    try:
        # disable COREBlog2 standard output.
        import Products.COREBlog2
    except:
        pass


def test_suite():
    from Testing.ZopeTestCase import ZopeDocTestSuite
    from Testing.ZopeTestCase import ZopeDocFileSuite
    from unittest import TestSuite
    return TestSuite((
        # DocTest in the self-module is registered as TestSuite.
        ZopeDocTestSuite(setUp=setUp),

        # DocTest of a specified file is registered as TestSuite.
        ZopeDocFileSuite('../content/smallcontent.py', setUp=setUp),
    ))


if __name__ == '__main__':
    framework()

