# -*- coding: utf-8 -*-
#$Id: testSetup.py 138 2006-05-08 16:02:40Z taka $
#############################################################################
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#
# ATCTSmallSample Setup tests
#

# It is testSetup.py to make with ATCTSmallSample first that tests the 
# processing of PloneQuickInstaller. There is a test class that confirms 
# whether ``installation processing`` and processing of 
# ``uninstallation`` was correctly done in the module respectively. The 
# test class has methods of the name that starts by ``test``. These each 
# method is called ``TestCase`` and group of TestCase is called 
# ``TestSuite``.
#
# Each TestCase is described testing code in Extentions/Install.py. The 
# following are concretely confirmed::
#
# - Was the contents type registered in portal_types?
# - Wes the contents type converted into DynamicViewFTI?
# - Is portal_factory effective to the contents type?
# - Was the skin under skins registered as skin layer?
# - Was the css object registered in portal_css?
#
# And, after this product is uninstalled, it confirms whether the 
# above-mentioned is all invalid.



# There are the following three lines for this module to call 
# framework.py as a code in this module when the unit is executed. Three 
# the following lines do not operate when executed from runalltest.py, 
# and test_suite() of this module is called.
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from AccessControl import Unauthorized
from Products.ATCTSmallSample.tests.ATCTSSTestCase import ATCTSSTestCase
from Products.ATCTSmallSample.config import *

# The test suite that confirms whether to complete the installation 
# correctly. The preparation for the test is completed only by importing 
# and inheriting ATCTSSTestCase because zope/plone installation 
# processing etc. are written in the code of the ATCTSSTestCase module.
class TestInstallSmallContent(ATCTSSTestCase):

    # The registration of portal_type is confirmed. When the test fails, 
    # DocString is output instead of the failing test function name if 
    # DocString is written in the test method.
    def testSmallContentPortalType(self):
        """SmallContent in portal_types"""
        self.failUnless('SmallContent' in self.portal.portal_types.objectIds())

    # Whether SmallContent has been converted into DynamicViewFTI is 
    # confirmed. The presence of interface implementation is examined for 
    # the confirmation.
    def testSmallContentDynamicViewFTI(self):
        """SmallContent implemented by IDynamicViewTypeInformation"""
        small = getattr(self.folder, 'small')
        from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
        info = self.portal.portal_types['SmallContent']
        self.failUnless(IDynamicViewTypeInformation.isImplementedBy(info))

    # Whether portal_factory is effective is confirmed. Because 
    # portal_factory is made effective in not the entire product but each 
    # contents type, each contents type made effective should test.
    def testSmallContentPortalFactoryEnabled(self):
        """PortalFactory enabled on SmallContent"""
        from Products.CMFCore.utils import getToolByName
        factory = getToolByName(self.portal, 'portal_factory')
        types = factory.getFactoryTypes().keys()
        self.failUnless('SmallContent' in types)

    # The skin layer registration to portal_skins is confirmed. 
    # ATCTSmallSample registers 'ATCTSmallSample' layer to all 
    # SkinSelection. Please note the difference about 'Registration of 
    # the layer' and 'FolderView registration'. ATCTSmallSample has only 
    # one skins directory under skin layer that name is the same as the 
    # product name. It is necessary for each layer name to do a similar 
    # test for the product that installs two or more skin layers.
    def testSmallContentSkin(self):
        """SmallContent skin path in portal_skins"""
        for selection in self.portal.portal_skins.getSkinSelections():
            path = self.portal.portal_skins.getSkinPath(selection)
            self.failUnless(PROJECTNAME in path)

    # The registration of the css object to portal_css is confirmed. Only 
    # the registration of smallcontent.css is confirmed here. However, 
    # you may confirm enable/disable.
    def testSmallContentPortalCSS(self):
        """smallcontent.css in portal_css"""
        cssreg = self.portal.portal_css
        stylesheet_ids = cssreg.getResourceIds()
        self.failUnless('smallcontent.css' in stylesheet_ids)


# This test confirms normal completion of uninstallation. A main purpose 
# of this test is a confirmation of the deletion of the skin layer. If 
# TOOL is installed in plone root at installation, it is necessary to 
# confirm the deletion of TOOL.
class TestUninstallSmallContent(ATCTSSTestCase):

    # This test confirms the deletion of the ATCTSmallSample skin layer. 
    # The problem doesn't occur when render Web page even if 
    # portal_skins has 'unknown skin layer'. But, 'unknown skin layer' 
    # will obstruct commit when you do the edit/save of the skin layer on 
    # the property page of portal_skins. The settlement is important. 
    # This test needs correcting for the product that registers two or 
    # more skin.
    def testSmallContentSkin(self):
        """SmallContent skin path not in portal_skins"""
        self.uninstallProduct(PROJECTNAME)
        for selection in self.portal.portal_skins.getSkinSelections():
            path = self.portal.portal_skins.getSkinPath(selection)
            self.failUnless(PROJECTNAME not in path)

    # In Plone-2.1.2, When the product is deleted, css registered in 
    # portal_css seems to be removed.
    def testSmallContentPortalCSS(self):
        """smallcontent.css not in portal_css"""
        self.uninstallProduct(PROJECTNAME)
        cssreg = self.portal.portal_css
        stylesheet_ids = cssreg.getResourceIds()
        self.failUnless('smallcontent.css' not in stylesheet_ids)


# This function registers the test suite is called from runalltests.py.
def test_suite():
    from unittest import TestSuite, makeSuite

    # Test suite stores two or more test suite and test case. 
    suite = TestSuite()

    # After converting the test class, I pass it to suite.addTest because 
    # suite.addTest cannot receive the TestCase class.
    suite.addTest(makeSuite(TestInstallSmallContent))
    suite.addTest(makeSuite(TestUninstallSmallContent))

    # I return test suite to the caller.
    return suite

if __name__ == '__main__':
    # framework() executes the ZopeTestCase inheritance class in this 
    # module as test suite. You need not register specifying it so that 
    # framework() may automatically recognize the test case class.
    framework()
